import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { fromJS } from 'immutable'
import { Modal, Button, Drawer, Tooltip } from 'antd'
import { ClockCircleOutlined, CheckCircleOutlined, CloseCircleOutlined, LoadingOutlined } from '@ant-design/icons'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import { formatSearch, joinMainSearchkey, getSearchRegs } from '@/utils/utils-custom.js'
import './index.scss'

class DebugSql extends Component {
  static propTpyes = {
    config: PropTypes.object
  }

  state = {
    visible: false,
    sqlList: [],
    status: '',
    successIds: [],
    errorIds: [],
    errorMsg: {},
    execId: ''
  }

  sqlList = []
  verSqls = []

  trigger = () => {
    let config = fromJS(this.props.config).toJS()

    let error = ''

    let check = (components) => {
      components.forEach(item => {
        if (error) return
        
        if (item.type === 'tabs') {
          item.subtabs.forEach(tab => {
            check(tab.components)
          })
          return
        } else if (item.type === 'group') {
          check(item.components)
          return
        } else if (!item.errors || item.errors.length === 0) {
          return
        }

        item.errors.forEach(err => {
          if (err.level !== 0 || error) return
          error = `组件《${item.name}》${err.detail}`
        })
      })
    }

    check(config.components)

    if (error) {
      Modal.warning({
        title: error,
        okText: '知道了'
      })
      return
    }

    this.sqlList = []

    let regs = [
      { reg: /@userName@/ig, value: `'User_Name'` },
      { reg: /@fullName@/ig, value: `'Full_Name'` },
      { reg: /\$@/ig, value: '' },
      { reg: /@\$/ig, value: '' },
      { reg: /@datam@/ig, value: `''` },
    ]
    
    if (window.GLOB.externalDatabase !== null) {
      regs.push({
        reg: /@db@/ig,
        value: window.GLOB.externalDatabase
      })
    }
    if (config.urlFields) {
      config.urlFields.forEach(field => {
        regs.push({
          reg: new RegExp('@' + field + '@', 'ig'),
          value: `'mk'`
        })
      })
    }

    let process = config.process === 'true'
    if (process) {
      regs.push({ reg: /@works_flow_code@/ig, value: `'1949-10-01 15:00:00'` })
    }
    
    if (config.interfaces && config.interfaces.length > 0) {
      config.interfaces.forEach(m => {
        if (m.status !== 'true' || m.setting.interType !== 'system') return false
  
        let sql = this.formatDataSource(m, regs)
  
        this.sqlList.push({label: m.setting.name, children: [{label: '数据源', sql: sql}]})
      })
    }

    this.filterComponent(config.components, [], regs, process)

    let sqls = []
    let foreachSql = (list, name, tabName = '', supName = '') => {
      list.forEach(item => {
        if (item.children) {
          if (item.tabName) {
            item.children.forEach(cell => {
              if (cell.children) {
                foreachSql(cell.children, cell.label, item.tabName, name)
              }
            })
          } else {
            foreachSql(item.children, name, tabName, supName)
          }
        } else if (item.sql) {
          sqls.push({uuid: Utils.getuuid() ,label: item.label, name: name, tabName: tabName, supName: supName, sql: item.sql})
        }
      })
    }

    this.sqlList.forEach(item => {
      if (item.children) {
        foreachSql(item.children, item.label)
      }
    })

    if (sqls.length === 0) {
      Modal.warning({
        title: '当前菜单无可验证脚本。',
        okText: '知道了'
      })
      return
    }

    this.verSqls = sqls
    let that = this

    Modal.confirm({
      content: `当前菜单共${this.sqlList.length}个组件，${sqls.length}项脚本需要检验${sqls.length > 20 ? '，时间大概需要' + parseInt(sqls.length / 2) + '秒' : ''}。`,
      onOk() {
        that.setState({visible: true, status: 'loading', sqlList: fromJS(sqls).toJS(), successIds: [], errorIds: [], errorMsg: {}, execId: ''}, () => {
          that.roopSql()
        })
      },
      onCancel() {}
    })

    this.sqlList = []
  }

  filterComponent = (components, mainSearch, regs, process, ispop = false) => {
    let appType = sessionStorage.getItem('appType')
    let _mainSearch = mainSearch || []

    if (appType === 'mob') {
      let search = []
      let ms = null
      components.forEach(item => {
        if (item.type === 'topbar' && item.wrap.type !== 'navbar' && item.search) {
          ms = item.search
        } else if (item.type === 'search' && item.wrap.field) {
          search.push({
            type: 'text',
            label: item.wrap.label,
            field: item.wrap.field,
            match: item.wrap.match,
            required: item.wrap.required,
            value: item.wrap.initval || ''
          })
        }
      })

      if (ms) {
        if (ms.setting.type === 'search') {
          search.push({
            type: 'text',
            label: '搜索栏',
            field: ms.setting.field,
            match: ms.setting.match,
            required: ms.setting.required,
            value: ms.setting.initval || ''
          })
        }
        search.push(...ms.fields)

        ms.groups.forEach(group => {
          if (group.setting.type === 'search') {
            search.push({
              type: 'text',
              label: group.wrap.name,
              field: group.setting.field,
              match: group.setting.match,
              required: group.setting.required,
              value: group.setting.initval || ''
            })
          }
          search.push(...group.fields)
        })
      }

      if (search.length > 0) {
        _mainSearch = search
      }
    } else {
      components.forEach(component => {
        if (component.type !== 'search') return
  
        _mainSearch = component.search || []
      })
    }

    components.forEach(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          this.filterComponent(tab.components, _mainSearch, regs, process)
        })
      } else if (item.type === 'group') {
        this.filterComponent(item.components, _mainSearch, regs, process)
      } else {
        let children = []
        if (item.wrap && item.setting) {
          if (item.wrap.datatype === 'public' || item.wrap.datatype === 'static') {
            item.setting.interType = 'other'
          }
        }

        if (appType === 'mob' && item.type !== 'search' && item.type !== 'topbar' && item.search && item.search.length > 0) {
          item.search = []
        }

        if (item.subtype === 'tablecard') { // 兼容
          item.type = 'card'
        }

        if (item.setting && item.setting.interType === 'system') {
          let sql = this.formatDataSource(item, regs, _mainSearch)
  
          children.push({label: '数据源', sql: sql})
        } else if (item.setting && item.setting.useMSearch === 'true') {
          let searches = item.search || []
          if (_mainSearch.length > 0) {
            searches = [...searches, ..._mainSearch]
          }
          item.$searches = fromJS(searches).toJS()
        }

        item.action && item.action.forEach(cell => {
          if (cell.hidden === 'true') return false
          let sql = this.resetButton(item, cell, process)

          if (sql) {
            if (typeof(sql) === 'string') {
              children.push({label: cell.label, sql: sql})
            } else {
              children.push({label: cell.label, tabName: cell.label, sql: '', children: sql})
            }
          }
        })

        if (item.type === 'table') {
          let getCols = (cols) => {
            cols.forEach(col => {
              if (col.type === 'action') {
                col.type = 'custom'
              }
              if (col.type === 'colspan') {
                getCols(col.subcols || [])
              } else if (col.type === 'custom') {
                col.elements.forEach(cell => {
                  if (cell.eleType !== 'button' || cell.hidden === 'true') return
                  let sql = this.resetButton(item, cell, process)

                  if (sql) {
                    if (typeof(sql) === 'string') {
                      children.push({label: cell.label, sql: sql})
                    } else {
                      children.push({label: cell.label, tabName: cell.label, sql: '', children: sql})
                    }
                  }
                })
              }
            })
          }
          
          getCols(item.cols)
  
          if (item.subtype === 'editable' && item.submit.intertype === 'system') {
            let sql = this.getEditTableSql(item.submit, item.cols, item.columns)
            children.push({label: '提交', sql: sql})
          }
        } else if (item.type === 'card' || item.type === 'carousel' || item.type === 'timeline') {
          item.subcards && item.subcards.forEach(card => {
            card.elements && card.elements.forEach(cell => {
              if (cell.eleType !== 'button' || cell.hidden === 'true') return
              let sql = this.resetButton(item, cell, process)

              if (sql) {
                if (typeof(sql) === 'string') {
                  children.push({label: cell.label, sql: sql})
                } else {
                  children.push({label: cell.label, tabName: cell.label, sql: '', children: sql})
                }
              }
            })
  
            if (!card.backElements || card.backElements.length === 0) return
  
            card.backElements.forEach(cell => {
              if (cell.eleType !== 'button' || cell.hidden === 'true') return
              let sql = this.resetButton(item, cell, process, null, true)

              if (sql) {
                if (typeof(sql) === 'string') {
                  children.push({label: cell.label, sql: sql})
                } else {
                  children.push({label: cell.label, tabName: cell.label, sql: '', children: sql})
                }
              }
            })
          })
        } else if (item.type === 'balcony') {
          item.elements.forEach(cell => {
            if (cell.eleType !== 'button' || cell.hidden === 'true') return
            let sql = this.resetButton(item, cell, process)

            if (sql) {
              if (typeof(sql) === 'string') {
                children.push({label: cell.label, sql: sql})
              } else {
                children.push({label: cell.label, tabName: cell.label, sql: '', children: sql})
              }
            }
          })
        } else if (item.type === 'form') {
          item.subcards.forEach(group => {
            group.subButton.OpenType = 'formSubmit'
            if (!group.subButton.Ot) {
              group.subButton.Ot = item.wrap.datatype === 'static' ? 'notRequired' : 'requiredSgl'
            }
            let sql = this.resetButton(item, group.subButton, process, group)

            if (sql) {
              children.push({label: group.subButton.label, sql: sql})
            }
          })
        }

        if (children.length) {
          if (ispop) {
            this.sqlPopList.push({label: item.name, children: children})
          } else {
            this.sqlList.push({label: item.name, children: children})
          }
        }
      }
    })
  }

  resetButton = (item, cell, process, group, isback) => {
    let sql = ''
    if (['exec', 'prompt', 'pop', 'form', 'formSubmit'].includes(cell.OpenType)) {
      if (cell.intertype === 'system' || cell.procMode === 'system') { // 系统接口
        if (item.subtype === 'dualdatacard' && isback) {
          let _item = fromJS(item).toJS()
          _item.columns = _item.subColumns || []
          _item.setting.primaryKey = _item.setting.subKey
          sql = this.getSysDefaultSql(cell, _item, process, group)
        } else {
          sql = this.getSysDefaultSql(cell, item, process, group)
        }
      }
    } else if (cell.OpenType === 'excelIn') {
      if (cell.intertype === 'system') {
        sql = this.getExcelInSql(cell)
      }
    } else if (cell.OpenType === 'excelOut') {
      if (cell.intertype === 'system' && cell.verify && cell.verify.dataType === 'custom') {
        sql = this.getExcelOutSql(cell, item)
      }
    } else if (cell.OpenType === 'funcbutton') {
      if (cell.funcType === 'print') {

      }
    } else if (cell.OpenType === 'popview') {
      if (cell.config && cell.config.components) {
        this.sqlPopList = []
        let regs = [
          { reg: /@userName@/ig, value: `'User_Name'` },
          { reg: /@fullName@/ig, value: `'Full_Name'` },
          { reg: /\$@/ig, value: '' },
          { reg: /@\$/ig, value: '' },
          { reg: /@datam@/ig, value: `''` },
        ]
        
        if (window.GLOB.externalDatabase !== null) {
          regs.push({
            reg: /@db@/ig,
            value: window.GLOB.externalDatabase
          })
        }
    
        if (process) {
          regs.push({ reg: /@works_flow_code@/ig, value: `'1949-10-01 15:00:00'` })
        }
        this.filterComponent(cell.config.components, [], regs, process, true)

        if (this.sqlPopList.length) {
          sql = fromJS(this.sqlPopList).toJS()
        }
      }
    }

    return sql
  }

  getSysDefaultSql = (btn, component, process, group) => {
    let primaryId = 'id'
    let BID = 'bid'
    let verify = btn.verify || {}
    let _actionType = null
    let setting = component.setting
    let columns = component.columns || []
  
    if (verify.default !== 'false') { // 判断是否使用默认sql
      _actionType = btn.sqlType
    }
  
    let _initCustomScript = '' // 初始化脚本
    let _prevCustomScript = '' // 默认sql前执行脚本
    let _backCustomScript = '' // 默认sql后执行脚本
  
    verify.scripts && verify.scripts.forEach(item => {
      if (item.status === 'false') return
  
      if (item.position === 'init') {
        _initCustomScript += `
        /* 自定义脚本 */
        ${item.sql}
        `
      } else if (item.position === 'front') {
        _prevCustomScript += `
        /* 自定义脚本 */
        ${item.sql}
        `
      } else {
        _backCustomScript += `
        /* 自定义脚本 */
        ${item.sql}
        `
      }
    })
  
    // 需要声明的变量集
    let _vars = ['tbid', 'errorcode', 'retmsg', 'billcode', 'bvoucher', 'fibvoucherdate', 'fiyear', 'username', 'fullname', 'modulardetailcode', 'roleid', 'mk_departmentcode', 'mk_organization', 'mk_user_type', 'mk_nation', 'mk_province', 'mk_city', 'mk_district', 'mk_address', 'mk_deleted', 'bid']
  
    // 主键字段
    let primaryKey = setting.primaryKey || 'id'
  
    // sql语句
    let _sql = ''
  
    let _initvars = [] // 已赋值字段集
    let _initFormfields = []
    let _initColfields = []
    let _declarefields = []

    let formdata = null
    if (btn.OpenType === 'pop') {
      formdata = []
      if (btn.modal && btn.modal.fields) {
        btn.modal.fields.forEach(item => {
          if (!item.field) return
          
          let _item = {
            key: item.field,
            fieldlen: item.fieldlength || 50,
            writein: item.writein !== 'false',
            type: item.type
          }
    
          if (_item.type === 'datemonth') {
            _item.type = 'text'
          } else if (_item.type === 'number' || _item.type === 'rate') {
            _item.fieldlen = item.decimal || 0
          } else  if (_item.type === 'date') {
            _item.type = item.declareType === 'nvarchar(50)' ? 'text' : 'date'
          }
    
          formdata.push(_item)
        })
      }
    } else if (btn.OpenType === 'form') {
      formdata = []

      let item = {
        type: 'text',
        readin: true,
        writein: true,
        fieldlen: 50,
        key: btn.field
      }
      if (btn.formType === 'counter') {
        item.type = 'number'
        item.fieldlen = 0
      } else if (btn.formType === 'switch' || btn.formType === 'radio') {
        if (typeof(btn.openVal) === 'number') {
          item.type = 'number'
          item.fieldlen = 0
        }
      }
      formdata.push(item)
    } else if (btn.OpenType === 'formSubmit') {
      formdata = []
      if (group.fields) {
        group.fields.forEach(item => {
          if (!item.field) return
          
          let _item = {
            key: item.field,
            fieldlen: item.fieldlength || 50,
            writein: item.writein !== 'false',
            type: item.type
          }
    
          if (_item.type === 'datemonth') {
            _item.type = 'text'
          } else if (_item.type === 'number' || _item.type === 'rate') {
            _item.fieldlen = item.decimal || 0
          } else  if (_item.type === 'date') {
            _item.type = item.declareType === 'nvarchar(50)' ? 'text' : 'date'
          }
    
          formdata.push(_item)
        })
      }
    }
    // 获取字段键值对
    formdata && formdata.forEach(form => {
      let _key = form.key.toLowerCase()
      if (!_initvars.includes(_key)) {
        _initvars.push(_key)
        if (form.type === 'number' || form.type === 'rate') {
          _initFormfields.push(`@${_key}=1`)
        } else if (form.type === 'date') {
          _initFormfields.push(`@${_key}='1949-10-01'`)
        } else if (form.type === 'select' || form.type === 'link' || form.type === 'radio') {
          _initFormfields.push(`@${_key}='1'`)
        } else {
          _initFormfields.push(`@${_key}='mk'`)
        }
      }
      
      if (!_vars.includes(_key)) {
        _vars.push(_key)
  
        if (form.fieldlen && form.fieldlen > 4000) {
          form.fieldlen = 'max'
        }
  
        let _type = `nvarchar(${form.fieldlen})`
  
        if (form.type.match(/date/ig)) {
          _type = 'datetime'
        } else if (form.type === 'number') {
          _type = `decimal(18,${form.fieldlen})`
        } else if (form.type === 'rate') {
          _type = `decimal(18,2)`
        }
  
        _declarefields.push(`@${_key} ${_type}`)
      }
    })
  
    // 添加数据中字段，表单值优先(按钮不选行或多行拼接时跳过)
    if (btn.Ot !== 'notRequired' && columns && columns.length > 0) {
      const setField = (col) => {
        if (!col.field) return
        let _key = col.field.toLowerCase()
  
        if (!_initvars.includes(_key)) {
          _initvars.push(_key)
  
          if (col.datatype && /^date/ig.test(col.datatype)) {
            _initColfields.push(`@${_key}='1949-10-01'`)
          } else if (col.type === 'number') {
            _initColfields.push(`@${_key}=1`)
          } else {
            _initColfields.push(`@${_key}='mk'`)
          }
        }
        
        if (!_vars.includes(_key)) {
          _vars.push(_key)
  
          if (col.datatype) {
            _declarefields.push(`@${_key} ${col.datatype}`)
          } else {
            if (col.fieldlength && col.fieldlength > 4000) {
              col.fieldlength = 'max'
            }
  
            let _type = `nvarchar(${col.fieldlength || 50})`
  
            if (col.type === 'number') {
              let _length = col.decimal ? col.decimal : 0
              _type = `decimal(18,${_length})`
            } else if (col.type === 'picture' || col.type === 'textarea') {
              _type = `nvarchar(${col.fieldlength || 512})`
            }
  
            _declarefields.push(`@${_key} ${_type}`)
          }
        }
      }
  
      columns.forEach(col => {
        if (col.type === 'colspan' || col.type === 'old_colspan') {
          col.subcols.forEach(cell => {
            setField(cell)
          })
        } else {
          setField(col)
        }
      })
    }
  
    // 变量声明
    _declarefields = _declarefields.join(',')
    if (_declarefields) {
      _declarefields = ',' + _declarefields
    }
    _sql = `Declare @tbid nvarchar(50),@ErrorCode nvarchar(50),@retmsg nvarchar(4000),@BillCode nvarchar(50),@BVoucher nvarchar(50),@FIBVoucherDate nvarchar(50), @FiYear nvarchar(50),@ModularDetailCode nvarchar(50), @UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100),@mk_deleted int,@bid nvarchar(50)${_declarefields}
      `
  
    let userName = 'User_Name'
    let fullName = 'Full_Name'
    let RoleID = 'role_id'
    let departmentcode = 'departmentcode'
    let organization = 'organization'
    let mk_user_type = 'mk_user_type'
    let nation = 'nation'
    let province = 'province'
    let city = 'city'
    let district = 'district'
    let address = 'address'
  
    // 初始化凭证及用户信息字段
    _sql += `
        /* 凭证及用户信息初始化赋值 */
        select @BVoucher='',@FIBVoucherDate='',@FiYear='',@ErrorCode='',@retmsg='',@UserName='${userName}', @FullName='${fullName}', @RoleID='${RoleID}', @mk_departmentcode='${departmentcode}', @mk_organization='${organization}', @mk_user_type='${mk_user_type}', @mk_nation='${nation}', @mk_province='${province}', @mk_city='${city}', @mk_district='${district}', @mk_address='${address}', @mk_deleted=1, @bid='${BID}', @BillCode='', @ModularDetailCode=''
        `
  
    // 表单变量赋值
    if (_initFormfields.length > 0) {
      _sql += `
        /* 表单变量赋值 */
        select ${_initFormfields.join(',')}
        `
    }
    // 显示列变量赋值
    if (_initColfields.length > 0) {
      _sql += `
        /* 显示列变量赋值 */
        select ${_initColfields.join(',')}
        `
    }
  
    // 去除禁用的验证
    if (verify.contrasts) {
      verify.contrasts = verify.contrasts.filter(item => item.status !== 'false')
    }
    if (verify.uniques) {
      verify.uniques = verify.uniques.filter(item => item.status !== 'false')
    }
    if (verify.customverifys) {
      verify.customverifys = verify.customverifys.filter(item => item.status !== 'false')
    }
    if (verify.billcodes) {
      verify.billcodes = verify.billcodes.filter(item => item.status !== 'false')
    }
  
    if (_initCustomScript) {
      _sql += _initCustomScript
    }
  
    // 启用账期验证
    if (verify.accountdate === 'true') {
      let orgcode = `''`
      let date = `''`
      if (verify.accountfield && _initvars.includes(verify.accountfield.toLowerCase())) {
        orgcode = '@' + verify.accountfield
      }
      if (verify.voucherdate && _initvars.includes(verify.voucherdate.toLowerCase())) {
        date = '@' + verify.voucherdate
      }
  
      _sql += `
        /* 账期验证 */
        exec s_FIBVoucherDateCheck @OrgCode=${orgcode},@FIBVoucherDate=${date},@ErrorCode=@ErrorCode OUTPUT,@retmsg=@retmsg OUTPUT
        if @ErrorCode!=''
          GOTO aaa
        `
    }
  
    // 失效验证，添加数据时不用
    if (btn.sqlType !== 'insert' && btn.Ot !== 'notRequired' && verify.invalid === 'true' && setting.dataresource) {
      let datasource = setting.dataresource
      let customScript = setting.customScript || ''
      let orderBy = setting.order

      if (setting.queryType === 'statistics' || customScript) {
        let searches = formatSearch(component.$searches || [])
        let regoptions = getSearchRegs(searches)
  
        regoptions.push({
          reg: new RegExp('@userName@', 'ig'),
          value: `'${userName}'`
        }, {
          reg: new RegExp('@fullName@', 'ig'),
          value: `'${fullName}'`
        }, {
          reg: new RegExp('@orderBy@', 'ig'),
          value: orderBy
        }, {
          reg: new RegExp('@pageSize@', 'ig'),
          value: 999999
        }, {
          reg: new RegExp('@pageIndex@', 'ig'),
          value: 1
        })
  
        regoptions.forEach(item => {
          datasource = datasource.replace(item.reg, item.value)
          customScript = customScript.replace(item.reg, item.value)
        })
      }
  
      if (customScript) {
        _sql += `
        /* 数据源自定义脚本，请注意变量定义是否重复 */
        ${customScript}
        `
      }
  
      if (btn.Ot === 'requiredOnce') {
        _sql += `
        /* 失效验证 */
        select @tbid='', @ErrorCode='',@retmsg=''
        select @tbid='X' from ${datasource} right join (select ID from  dbo.SplitComma(@ID@)) sp
        on tb.${primaryKey} =sp.id where tb.${primaryKey} is null
  
        If @tbid!=''
        Begin
          select @ErrorCode='E',@retmsg='数据已失效'
          goto aaa
        end
        `
      } else {
        _sql += `
        /* 失效验证 */
        select @tbid='', @ErrorCode='',@retmsg=''
        select @tbid=${primaryKey} from ${datasource} where ${primaryKey}=@ID@
        If @tbid=''
        Begin
          select @ErrorCode='E',@retmsg='数据已失效'
          goto aaa
        end
        `
      }
    }
  
    // 比较验证
    if (verify.contrasts && verify.contrasts.length > 0) {
      verify.contrasts.forEach(item => {
        _sql += `
        /* 比较验证 */
        If ${item.frontfield} ${item.operator} ${item.backfield}
        Begin
          select @ErrorCode='${item.errorCode}',@retmsg='${item.errmsg}'
            goto aaa
        end
        `
      })
    }
    
    // 自定义验证
    verify.customverifys && verify.customverifys.forEach(item => {        
      _sql += `
        /* 自定义验证 */
        select @tbid='', @ErrorCode='',@retmsg=''
        select top 1 @tbid='X' from (${item.sql}) a
        If @tbid ${item.result === 'true' ? '!=' : '='}''
        Begin
          select @ErrorCode='${item.errorCode}',@retmsg='${item.errmsg}'
          goto aaa
        end
        `
    })
  
    // 单号生成，使用上级id（BID）或列表数据，声明变量（检验）
    let _billcodesSql  = ''
    if (formdata && verify.billcodes && verify.billcodes.length > 0) {
      let keys = formdata.map(item => item.key.toLowerCase()) // 表单字段
  
      verify.billcodes.forEach(item => {
        let _key = item.field.toLowerCase()
        let _linkKey = item.linkField ? item.linkField.toLowerCase() : ''
  
        if (!keys.includes(_key)) return // 表单中不含单号生成字段
  
        let _ModularDetailCode = ''
        let _lpline = ''
        if (item.TypeCharOne === 'Lp') {
          if (_linkKey === 'bid' && BID) { // 替换bid
            _lpline = `set @ModularDetailCode= 'Lp'+ right('${item.mark || btn.uuid}'+@BID@,48)`
          } else {
            _lpline = `set @ModularDetailCode= 'Lp'+ right('${item.mark || btn.uuid}'+@${_linkKey},48)`
          }
          _ModularDetailCode = '@ModularDetailCode'
        } else if (item.TypeCharOne === 'BN') {
          let _val = ''
          if (_linkKey === 'bid' && BID) { // 替换bid
            _val = BID
          } else {
            _val = 0
          }
          _ModularDetailCode = `'${item.TypeCharOne + _val}'`
        } else {
          _ModularDetailCode = `'${item.ModularDetailCode}'`
        }
  
        let _declare = ''
  
        if (!_vars.includes(_key)) {
          _declare = `Declare @${_key} nvarchar(50)`
          _vars.push(_key)
        }
  
        _billcodesSql += `
        /* 单号生成 */
        ${_declare}
        select @BillCode='', @${_key}='', @ModularDetailCode=''
        ${_lpline}
        exec s_get_BillCode
          @ModularDetailCode=${_ModularDetailCode},
          @Type=${item.Type},
          @TypeCharOne='${item.TypeCharOne}',
          @TypeCharTwo ='${item.TypeCharTwo}',
          @BillCode =@BillCode output,
          @ErrorCode =@ErrorCode output, 
          @retmsg=@retmsg output
        if @ErrorCode!=''
          goto aaa
        set @${_key}=@BillCode
        `
      })
  
      if (_actionType !== 'insertOrUpdate') {
        _sql += _billcodesSql
      }
    }
  
    // 唯一性验证，必须存在表单（表单存在时，主键均为单值）,必须填写数据源，多行拼接时不可用
    if (formdata && verify.uniques && verify.uniques.length > 0 && btn.Ot !== 'requiredOnce') {
      verify.uniques.forEach(item => {
        let _fieldValue = []                     // 表单键值对field=value
        let _value = []                          // 表单值，用于错误提示
        let _labels = item.fieldlabel.split(',') // 表单提示文字
        let arr = [] // 验证主键
  
        item.field.split(',').forEach((_field, index) => {
          let _key = _field.toLowerCase()
          let _val = ''
          let _fval = `'${_val}'`
  
          if (_key === 'bid') { // 表单中没有bid则使用系统bid变量
            _fval = '@BID@'
          }

          arr.push(_key)
          _fieldValue.push(`${_key}=${_fval}`)
          
          _value.push(`${_labels[index] || ''}：${_val || ''}`)
        })
  
        let _verifyType = ''
        if (item.verifyType === 'logic') {
          _verifyType = ' and deleted=0'
        }
  
        if (!arr.includes(primaryKey.toLowerCase())) {
          _fieldValue.push(`${primaryKey} !='${primaryId}'`)
        }
  
        _sql += `
        /* 唯一性验证 */
        select @tbid='', @ErrorCode='',@retmsg=''
        select @tbid='X' from ${btn.sql} where ${_fieldValue.join(' and ')}${_verifyType}
        If @tbid!=''
        Begin
          select @ErrorCode='${item.errorCode}',@retmsg='${_value.join(', ')} 已存在'
          goto aaa
        end
        `
      })
    } else if (verify.uniques && verify.uniques.length > 0 && btn.Ot === 'requiredOnce' && setting.dataresource) {
      let datasource = setting.dataresource
      if (/\s/.test(datasource)) { // 拼接别名
        if (!/tb$/.test(datasource)) {
          datasource = '(' + datasource + ') tb'
        }
      } else {
        datasource = datasource + ' tb'
      }
  
      if (setting.customScript) {
        _sql += `
        /* 数据源自定义脚本，请注意变量定义是否重复 */
        ${setting.customScript}
        `
      }
  
      verify.uniques.forEach(item => {
        _sql += `
        /* 同类数据验证 */
        Set @tbid=''
  
        Select top 1 @tbid='X' from (select distinct ${item.field},1 as n from ${datasource} inner join (select ID from  dbo.SplitComma(@ID@)) sp on tb.${primaryKey}=sp.ID ) a having sum(n)>1
        
        If @tbid!=''
        Begin
          Set @ErrorCode='E' Set @retmsg='${item.fieldlabel} 值不唯一'
          goto aaa
        end
        `
      })
    }
  
    let hasvoucher = false
  
    // 凭证-显示列中选取,必须选行
    if (verify.voucher && verify.voucher.enabled && btn.Ot !== 'requiredOnce') {
      let _voucher = verify.voucher
  
      hasvoucher = true
  
      _sql += `
        /* 创建凭证 */
        exec s_BVoucher_Create
          @Bill ='0',
          @BVoucherType ='${_voucher.BVoucherType}',
          @VoucherTypeOne ='${_voucher.VoucherTypeOne}',
          @VoucherTypeTwo ='${_voucher.VoucherTypeTwo}',
          @Type =${_voucher.Type},
          @UserID=@UserID@,
          @Username=@Username,
          @FullName=@FullName,
          @BVoucher =@BVoucher OUTPUT ,
          @FIBVoucherDate =@FIBVoucherDate OUTPUT ,
          @FiYear =@FiYear OUTPUT ,
          @ErrorCode =@ErrorCode OUTPUT, 
          @retmsg=@retmsg OUTPUT
        if @ErrorCode!=''
          GOTO aaa
        `
    }
  
    let _insertsql = ''
    if (_actionType === 'insert' || _actionType === 'insertOrUpdate') { // 添加语句
      let keys = []
      let values = []
  
      formdata.forEach(item => {
        if (item.writein === false) return
        let _key = item.key.toLowerCase()
  
        keys.push(_key)
        values.push('@' + _key)
      })
  
      if (!keys.includes(primaryKey.toLowerCase())) {
        keys.push(primaryKey.toLowerCase())
        values.push('\'' + primaryId + '\'')
      }
      if (!keys.includes('createuserid')) {
        keys.push('createuserid')
        values.push('@userid@')
      }
      if (!keys.includes('createuser')) {
        keys.push('createuser')
        values.push('@username')
      }
      if (!keys.includes('createstaff')) {
        keys.push('createstaff')
        values.push('@fullname')
      }
      if (!keys.includes('bid')) {
        keys.push('bid')
        values.push('@BID@')
      }
  
      if (!keys.includes('typename')) {
        keys.push('typename')
        values.push('@typename@')
      }
  
      keys = keys.join(',')
      values = values.join(',')
      _insertsql = `insert into ${btn.sql} (${keys}) select ${values};`
    }
  
    let _updatesql = ''
    if (_actionType === 'update' || _actionType === 'audit' || _actionType === 'insertOrUpdate') { // 修改语句
      let _form = []
      let _arr = []
  
      formdata.forEach(item => {
        if (item.writein === false) return
        let _key = item.key.toLowerCase()
        
        _arr.push(_key)
        _form.push(_key + '=@' + _key)
      })
  
      if (_actionType === 'audit') {
        if (!_arr.includes('submitdate')) {
          _form.push('submitdate=getdate()')
        }
        if (!_arr.includes('submituser')) {
          _form.push('submituser=@username')
        }
        if (!_arr.includes('submitstaff')) {
          _form.push('submitstaff=@fullname')
        }
        if (!_arr.includes('submituserid')) {
          _form.push('submituserid=@userid@')
        }
      } else {
        if (!_arr.includes('modifydate')) {
          _form.push('modifydate=getdate()')
        }
        if (!_arr.includes('modifyuser')) {
          _form.push('modifyuser=@username')
        }
        if (!_arr.includes('modifystaff')) {
          _form.push('modifystaff=@fullname')
        }
        if (!_arr.includes('modifyuserid')) {
          _form.push('modifyuserid=@userid@')
        }
      }
      
      if (hasvoucher) {
        if (!_arr.includes('bvoucher')) {
          _form.push('BVoucher=@BVoucher')
        }
        if (!_arr.includes('fibvoucherdate')) {
          _form.push('FIBVoucherDate=@FIBVoucherDate')
        }
        if (!_arr.includes('fiyear')) {
          _form.push('FiYear=@FiYear')
        }
      }
      if (!_arr.includes('typename')) {
        _form.push('typename=@typename@')
      }
      _form = _form.join(',')
  
      let _ID = '=@ID@'
      if (btn.Ot === 'requiredOnce') {
        _ID = ' in (select ID from  dbo.SplitComma(@ID@))'
      }
  
      _updatesql = `update ${btn.sql} set ${_form} where ${primaryKey}${_ID};`
    }
  
    if (_prevCustomScript) {
      _sql += _prevCustomScript
    }
  
    // 添加、修改、逻辑删除、物理删除
    if (_actionType === 'insert') {
      _sql += `
        /* 默认sql */
        ${_insertsql}`
    } else if (_actionType === 'update' || _actionType === 'audit') {
      _sql += `
        /* 默认sql */
        ${_updatesql}`
    } else if (_actionType === 'LogicDelete') { // 逻辑删除
      let _ID = '=@ID@'
      if (btn.Ot === 'requiredOnce') {
        _ID = ' in (select ID from  dbo.SplitComma(@ID@))'
      }
  
      _sql += `
        /* 默认sql */
        update ${btn.sql} set deleted=@mk_deleted,modifydate=getdate(),modifyuser=@username,modifystaff=@fullname,modifyuserid=@userid@ where ${primaryKey}${_ID};`
    
    } else if (_actionType === 'delete') {      // 物理删除
      let _msg = ''
      if (columns && columns.length > 0 && btn.Ot !== 'notRequired') {
        let _index = 0
        columns.forEach(col => {
          if (!col.field || col.Hide === 'true' || _index >= 4) return
          _msg += col.label + '=0,'
          _index++
        })
      }
  
      let _ID = '=@ID@'
      if (btn.Ot === 'requiredOnce') {
        _ID = ' in (select ID from  dbo.SplitComma(@ID@))'
      }
  
      _sql += `
        /* 默认sql */
        insert into snote (remark,createuserid,CreateUser,CreateStaff,typename) select left('删除表:${btn.sql} 数据: ${_msg}${primaryKey}='+@ID@,200),@userid@,@username,@fullname,@typename@
        delete ${btn.sql} where ${primaryKey}${_ID};`
    } else if (_actionType === 'insertOrUpdate') {
      _sql += `
        /* 默认sql */
        select @tbid=''
        select @tbid='X' from ${btn.sql} where ${primaryKey}=@ID@
        if @tbid=''
          begin
          ${_billcodesSql}
          ${_insertsql}
          end
        else
          begin
          ${_updatesql}
          end
      `
    }
  
    if (verify.workFlow === 'true' && process) {
      let status = 888
      let statusName = '结束'
      let detailId = '0'
  
      if (verify.flowSql === 'true') {
        _sql += `
        /* 工作流默认sql */
        insert into s_my_works_flow (works_flow_id,works_flow_code,works_flow_name,works_flow_param,status,statusname,work_group,works_flow_detail_id,work_grade,bid,createuserid,CreateUser,CreateStaff,upid)
        select @ID@,@works_flow_code@,@works_flow_name@,@works_flow_param@,@status@,@statusname@,@work_group@,@works_flow_detail_id@,@work_grade@,@bid@,@UserID@,@UserName,@FullName,@time_id@
        insert into s_my_works_flow_log (works_flow_id,works_flow_code,works_flow_name,works_flow_param,status,statusname,works_flow_detail_id,work_group,work_grade,upid)
        select @ID@,@works_flow_code@,@works_flow_name@ ,@works_flow_param@,@status@,@statusname@,@works_flow_detail_id@,@work_group@,@work_grade@,@time_id@
        insert into s_my_works_flow_notice (works_flow_id,works_flow_code,works_flow_detail_id,userid,notice_type,createuserid,CreateUser,CreateStaff,upid)
        select @ID@,@works_flow_code@,@works_flow_detail_id@,@userid@,@start_type@,@userid@,@UserName,@FullName,@time_id@
        `
      }
  
      _sql = _sql.replace(/@start_type@/ig, `'开始'`)
      _sql = _sql.replace(/@check_type@/ig, `'审核'`)
      _sql = _sql.replace(/@notice_type@/ig, `'抄送'`)
      _sql = _sql.replace(/@check_userids@/ig, `''`)
      _sql = _sql.replace(/@notice_userids@/ig, `''`)
      _sql = _sql.replace(/@works_flow_code@/ig, `'mk'`)
      _sql = _sql.replace(/@works_flow_name@/ig, `'mk'`)
      _sql = _sql.replace(/@works_flow_param@/ig, `''`)
      _sql = _sql.replace(/@works_flow_detail_id@/ig, `'${detailId}'`)
      _sql = _sql.replace(/@status@/ig, `'${status}'`)
      _sql = _sql.replace(/@statusname@/ig, `'${statusName}'`)
      _sql = _sql.replace(/@work_group@/ig, `'mk'`)
      _sql = _sql.replace(/@work_grade@/ig, `'0'`)
    }
  
    if (_backCustomScript) {
      _sql += _backCustomScript
    }
  
    if (btn.procMode === 'system') {
      _sql += `
        aaa: if @ErrorCode!=''
        insert into tmp_err_retmsg (ID, ErrorCode, retmsg, CreateUserID) select @time_id@,@ErrorCode, @retmsg,@UserID@`
    } else if (btn.output) {
      _sql += `
        aaa: select @ErrorCode as ErrorCode,@retmsg as retmsg,${btn.output} as mk_b_id`
    } else {
      _sql += `
        aaa: select @ErrorCode as ErrorCode,@retmsg as retmsg`
    }
  
    _sql = _sql.replace(/@ID@/ig, `'${primaryId || ''}'`)
    _sql = _sql.replace(/@BID@/ig, `'${BID}'`)
    _sql = _sql.replace(/@LoginUID@/ig, `'LoginUID'`)
    _sql = _sql.replace(/@SessionUid@/ig, `'SessionUid'`)
    _sql = _sql.replace(/@UserID@/ig, `'UserID'`)
    _sql = _sql.replace(/@Appkey@/ig, `'appkey'`)
    _sql = _sql.replace(/@typename@/ig, `'typename'`)
    _sql = _sql.replace(/@\$|\$@/ig, '').replace(/@datam@/ig, `''`)
  
    return _sql
  }

  getExcelInSql = (item) => {
    let btn = item.verify
    let userName = 'User_Name'
    let fullName = 'Full_Name'
    let RoleID = 'role_id'
    let departmentcode = 'departmentcode'
    let organization = 'organization'
    let mk_user_type = 'mk_user_type'
    let nation = 'nation'
    let province = 'province'
    let city = 'city'
    let district = 'district'
    let address = 'address'
    let _sheet = item.sheet
  
    if (window.GLOB.externalDatabase !== null) {
      _sheet = _sheet.replace(/@db@/ig, window.GLOB.externalDatabase)
    }
  
    let database = _sheet.match(/(.*)\.(.*)\./ig)
    let sheet = _sheet.replace(/(.*)\.(.*)\./ig, '')
    
    database = database ? (database[0] || '') : ''
  
    let getuuid = () => {
      let uuid = []
      let timestamp = new Date().getTime()
      let _options = '0123456789abcdefghigklmnopqrstuv'
      for (let i = 0; i < 19; i++) {
        uuid.push(_options.substr(Math.floor(Math.random() * 0x20), 1))
      }
      uuid = timestamp + uuid.join('')
      return uuid
    }

    let upId = getuuid()
  
    let vals = []
    btn.columns.forEach(col => {
      if (col.import === 'false') return

      let val = ''
      if (col.import === 'init') {
        if (/^Nvarchar/ig.test(col.type)) {
          val = ''
        } else if (/^Decimal/ig.test(col.type) || /^int/ig.test(col.type)) {
          val = 0
        } else if (col.type === 'date') {
          val = '1949-10-01'
        } else if (col.type === 'datetime') {
          val = '1949-10-01 00:00:00'
        }
      } else if (/^Nvarchar/ig.test(col.type)) {
        val = 'mk'
      } else if (/^Decimal/ig.test(col.type) || /^int/ig.test(col.type)) {
        val = 1
      } else if (col.type === 'date') {
        val = '1949-10-01'
      } else if (col.type === 'datetime') {
        val = '1949-10-01 00:00:00'
      }

      vals.push(`'${val}'`)
    })

    vals.push(`'${upId}000010'`)
    vals.push(`'bid'`)

    vals =  `Select ${vals.join(',')}`
  
    let sql = ''
    
    let _initCustomScript = '' // 初始化脚本
    let _prevCustomScript = '' // 默认sql前执行脚本
    let _backCustomScript = '' // 默认sql后执行脚本
    let regs = [
      {reg: new RegExp('(^|\\s)@' + sheet + '(\\s|$)', 'ig'), value: ` #${sheet} `},
      {reg: new RegExp('(^|\\s)@' + sheet + '\\(', 'ig'), value: ` #${sheet}(`},
      {reg: new RegExp('(^|\\s)@' + sheet + '\\)', 'ig'), value: ` #${sheet})`},
      {reg: /@ID@/ig, value: `'id'`},
      {reg: /@BID@/ig, value: `'bid'`},
      {reg: /@LoginUID@/ig, value: `'LoginUID'`},
      {reg: /@SessionUid@/ig, value: `'SessionUid'`},
      {reg: /@UserID@/ig, value: `'UserID'`},
      {reg: /@Appkey@/ig, value: `'appkey'`},
      {reg: /@typename@/ig, value: `'admin'`},
      {reg: /\$@/ig, value: ''},
      {reg: /@\$/ig, value: ''},
      {reg: /@datam@/ig, value: `''`},
    ]

    if (window.GLOB.externalDatabase !== null) {
      regs.push({reg: /@db@/ig, value: window.GLOB.externalDatabase})
    }

    btn.scripts && btn.scripts.forEach(script => {
      if (script.status === 'false') return

      let _sql = script.sql

      regs.forEach(item => {
        _sql = _sql.replace(item.reg, item.value)
      })

      if (script.position === 'init') {
        _initCustomScript += `
      /* 自定义脚本 */
      ${_sql}
      `
      } else if (script.position === 'front') {
        _prevCustomScript += `
      /* 自定义脚本 */
      ${_sql}
      `
      } else {
        _backCustomScript += `
      /* 自定义脚本 */
      ${_sql}
      `
      }
    })

    let _uniquesql = ''
    if (btn.uniques && btn.uniques.length > 0) {
      btn.uniques.forEach(unique => {
        if (unique.status === 'false' || !unique.verifyType) return

        let _fields = unique.field.split(',')
        let _fields_ = _fields.map(_field => `a.${_field}=b.${_field}`)
        let _afields = _fields.map(_field => `a.${_field}`)
        _fields_ = _fields_.join(' and ')

        if (unique.verifyType === 'logic' || unique.verifyType === 'logic_temp') {
          _fields_ += ' and b.deleted=0'
        }

        _uniquesql += `
      /* 重复性验证 */
      Set @tbid=''
      Select top 1 @tbid=${_fields.join('+\' \'+')} from (select 1 as n,${unique.field} from #${sheet} ) a group by ${unique.field} having sum(n)>1
      
      If @tbid!=''
      Begin
        select @ErrorCode='${unique.errorCode}',@retmsg=@tbid+' 重复'
        goto aaa
      end
      
      ${unique.verifyType.indexOf('temp') === -1 ? `Set @tbid=''
      Select top 1 @tbid=${_afields.join('+\' \'+')} from  #${sheet} a Inner join ${sheet} b on ${_fields_}
      
      If @tbid!=''
      Begin
        select @ErrorCode='${unique.errorCode}',@retmsg=@tbid+' 与已有数据重复'
        goto aaa
      end` : ''}
      `
      })
    }

    let declarefields = []
    let fields = []

    btn.columns.forEach(col => {
      if (col.import === 'false') return
      
      if (col.type === 'date') {
        declarefields.push(`${col.Column} Nvarchar(50)`)
      } else {
        declarefields.push(`${col.Column} ${col.type}`)
      }
      fields.push(col.Column)
    })

    fields = fields.join(',')

    let _insert = ''
    if (btn.default !== 'false') {
      _insert = `
      /* 默认sql */
      Insert into ${database}${sheet} (${fields},createuserid,createuser,createstaff,bid) 
      Select ${fields},'${sessionStorage.getItem('UserID') || ''}',@username,@fullname,'bid' From #${sheet}
      `
    }

    sql = `create table #${sheet} (${declarefields.join(',')},jskey nvarchar(50),BID nvarchar(50) )
      Declare @UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100),@ErrorCode nvarchar(50),@retmsg nvarchar(4000),@tbid Nvarchar(512)
      
      Select  @ErrorCode='', @retmsg='', @UserName='${userName}', @FullName='${fullName}', @RoleID='${RoleID}', @mk_departmentcode='${departmentcode}', @mk_organization='${organization}', @mk_user_type='${mk_user_type}', @mk_nation='${nation}', @mk_province='${province}', @mk_city='${city}', @mk_district='${district}', @mk_address='${address}'
      ${_initCustomScript}
      
      Insert into #${sheet} (${fields},jskey,BID)

      /* excel数据*/
      ${vals}

      ${_uniquesql}
      ${_prevCustomScript}
      ${_insert}
      ${_backCustomScript}

      drop table #${sheet}
      
      aaa: select @ErrorCode as ErrorCode,@retmsg as retmsg`

    return sql
  }

  getEditTableSql = (verify, cols, columns) => {
    let btn = verify
    let userName = sessionStorage.getItem('User_Name') || ''
    let fullName = sessionStorage.getItem('Full_Name') || ''
    let RoleID = sessionStorage.getItem('role_id') || ''
    let departmentcode = sessionStorage.getItem('departmentcode') || ''
    let organization = sessionStorage.getItem('organization') || ''
    let mk_user_type = sessionStorage.getItem('mk_user_type') || ''
    let nation = sessionStorage.getItem('nation') || ''
    let province = sessionStorage.getItem('province') || ''
    let city = sessionStorage.getItem('city') || ''
    let district = sessionStorage.getItem('district') || ''
    let address = sessionStorage.getItem('address') || ''
  
    let _sheet = btn.sheet
    let BID = 'bid'
  
    if (window.GLOB.externalDatabase !== null) {
      _sheet = _sheet.replace(/@db@/ig, window.GLOB.externalDatabase)
    }
  
    let database = _sheet.match(/(.*)\.(.*)\./ig)
    let sheet = _sheet.replace(/(.*)\.(.*)\./ig, '')
    
    database = database ? (database[0] || '') : ''
  
    let vals = []
    let forms = []
    let _fields = {}

    columns.forEach(col => {
      _fields[col.field] = col.datatype
    })

    let getColumns = (cols) => {
      cols.forEach(item => {
        if (item.type === 'colspan') {
          getColumns(item.subcols)
        } else if (item.editable === 'true') {
          forms.push({
            field: item.field,
            type: item.type,
            datatype: _fields[item.field] || 'nvarchar(50)'
          })
        }
      })
    }

    getColumns(cols)

    forms.forEach(col => {
      if (col.type === 'number') {
        vals.push(`1`)
      } else {
        vals.push(`'mk'`)
      }
    })

    vals.push(`'uuid'`)
    vals.push(`'upt'`)
    vals.push(`'${BID}'`)

    vals = `Select ${vals.join(',')}`
  
    let sql = ''
  
    let _initCustomScript = '' // 初始化脚本
    let _prevCustomScript = '' // 默认sql前执行脚本
    let _backCustomScript = '' // 默认sql后执行脚本
    let isDM = sessionStorage.getItem('dataM') === 'true'
    let regs = [
      {reg: new RegExp('(^|\\s)@' + sheet + '(\\s|$)', 'ig'), value: ` #${sheet} `},
      {reg: new RegExp('(^|\\s)@' + sheet + '\\(', 'ig'), value: ` #${sheet}(`},
      {reg: new RegExp('(^|\\s)@' + sheet + '\\)', 'ig'), value: ` #${sheet})`},
      {reg: /@BID@/ig, value: `'${BID}'`},
      {reg: /@LoginUID@/ig, value: `'${sessionStorage.getItem('LoginUID') || ''}'`},
      {reg: /@SessionUid@/ig, value: `'${localStorage.getItem('SessionUid') || ''}'`},
      {reg: /@UserID@/ig, value: `'${sessionStorage.getItem('UserID') || ''}'`},
      {reg: /@Appkey@/ig, value: `'${window.GLOB.appkey || ''}'`},
      {reg: /@typename@/ig, value: `'admin'`},
      {reg: /\$@/ig, value: isDM ? '/*' : ''},
      {reg: /@\$/ig, value: isDM ? '*/' : ''},
      {reg: /@datam@/ig, value: isDM ? `'Y'` : `''`},
    ]

    if (window.GLOB.externalDatabase !== null) {
      regs.push({reg: /@db@/ig, value: window.GLOB.externalDatabase})
    }

    btn.scripts && btn.scripts.forEach(script => {
      if (script.status === 'false') return

      let _sql = script.sql

      regs.forEach(item => {
        _sql = _sql.replace(item.reg, item.value)
      })

      if (script.position === 'init') {
        _initCustomScript += `
      /* 自定义脚本 */
      ${_sql}
      `
      } else if (script.position === 'front') {
        _prevCustomScript += `
      /* 自定义脚本 */
      ${_sql}
      `
      } else {
        _backCustomScript += `
      /* 自定义脚本 */
      ${_sql}
      `
      }
    })

    let _uniquesql = ''
    if (btn.uniques && btn.uniques.length > 0) {
      btn.uniques.forEach(unique => {
        if (unique.status === 'false' || !unique.verifyType) return

        let _fields = unique.field.split(',')
        let _fields_ = _fields.map(_field => `a.${_field}=b.${_field}`)
        let _afields = _fields.map(_field => `a.${_field}`)
        _fields_ = _fields_.join(' and ')

        if (unique.verifyType === 'logic' || unique.verifyType === 'logic_temp') {
          _fields_ += ' and b.deleted=0'
        }

        _uniquesql += `
      /* 重复性验证 */
      Set @tbid=''
      Select top 1 @tbid=${_fields.join('+\' \'+')} from (select 1 as n,${unique.field} from #${sheet} ) a group by ${unique.field} having sum(n)>1
      
      If @tbid!=''
      Begin
        select @ErrorCode='${unique.errorCode}',@retmsg=@tbid+' 重复'
        goto aaa
      end
      
      ${unique.verifyType.indexOf('temp') === -1 ? `Set @tbid=''
      Select top 1 @tbid=${_afields.join('+\' \'+')} from  #${sheet} a Inner join ${sheet} b on ${_fields_}
      
      If @tbid!=''
      Begin
        select @ErrorCode='${unique.errorCode}',@retmsg=@tbid+' 与已有数据重复'
        goto aaa
      end` : ''}
      `
      })
    }

    let declarefields = []
    let fields = []

    forms.forEach(col => {
      let key = col.field.toLowerCase()
      if (key === 'jskey' || key === 'bid' || key === 'data_type') return

      declarefields.push(`${col.field} ${col.datatype}`)
      fields.push(col.field)
    })

    fields = fields.join(',')

    let _insert = ''
    if (btn.default !== 'false') {
      _insert = `
      /* 默认sql */
      Insert into ${database}${sheet} (${fields},createuserid,createuser,createstaff,bid) 
      Select ${fields},'${sessionStorage.getItem('UserID') || ''}',@username,@fullname,'${BID}' From #${sheet}
      `
    }

    sql = `create table #${sheet} (${declarefields.join(',')},jskey nvarchar(50),data_type nvarchar(50),BID nvarchar(50) )
      Declare @UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100),@ErrorCode nvarchar(50),@retmsg nvarchar(4000),@tbid Nvarchar(512)
      
      Select  @ErrorCode='', @retmsg='', @UserName='${userName}', @FullName='${fullName}', @RoleID='${RoleID}', @mk_departmentcode='${departmentcode}', @mk_organization='${organization}', @mk_user_type='${mk_user_type}', @mk_nation='${nation}', @mk_province='${province}', @mk_city='${city}', @mk_district='${district}', @mk_address='${address}'
      ${_initCustomScript}
      Insert into #${sheet} (${fields},jskey,data_type,BID)

      /* excel数据*/
      ${vals}

      ${_uniquesql}
      ${_prevCustomScript}
      ${_insert}
      ${_backCustomScript}

      drop table #${sheet}
      
      aaa: select @ErrorCode as ErrorCode,@retmsg as retmsg`
  
    return sql
  }

  getExcelOutSql = (btn, component) => {
    let item = {setting: {}, columns: [], search: []}

    btn.verify.columns.forEach(col => {
      if (col.Column && col.Column !== '$Index') {
        item.columns.push({
          field: col.Column
        })
      }
    })

    if (btn.verify.useSearch !== 'false') {
      item.search = component.$searches
    }

    item.setting.interType = 'system'
    item.setting.execute = btn.verify.defaultSql || 'true'
    item.setting.dataresource = btn.verify.dataresource || ''
    item.setting.primaryKey = btn.verify.primaryKey || component.setting.primaryKey || 'ID'
    item.setting.queryType = btn.verify.queryType
    item.setting.laypage = btn.pagination
    item.setting.order = btn.verify.order || ''

    item.scripts = btn.verify.scripts || []

    let regs = [
      { reg: /\$@/ig, value: '' },
      { reg: /@\$/ig, value: '' },
      { reg: /@datam@/ig, value: `''` },
    ]
    
    if (window.GLOB.externalDatabase !== null) {
      regs.push({
        reg: /@db@/ig,
        value: window.GLOB.externalDatabase
      })
    }

    let sql = this.formatDataSource(item, regs)

    return sql
  }

  formatDataSource = (item, regs, mainSearch = []) => {
    if (!item.setting || item.setting.interType !== 'system') return false
    
    let searches = item.search || []
    if (item.setting.useMSearch === 'true' && mainSearch.length > 0) {
      searches = [...searches, ...mainSearch]
    }
    item.$searches = fromJS(searches).toJS()
    searches = formatSearch(searches)

    let _columns = item.columns || []
    if (item.subtype === 'dualdatacard') {
      _columns = [...item.columns, ...item.subColumns]
    }

    let arr_field = _columns.map(col => col.field).join(',')

    let _customScript = ''
    let _tailScript = ''
    let _dataresource = ''
    item.scripts && item.scripts.forEach(script => {
      if (script.status === 'false') return
      if (script.position !== 'back') {
        _customScript += `
        ${script.sql}
        `
      } else {
        _tailScript += `
        ${script.sql}
        `
      }
    })

    if (item.setting.execute !== 'false') {
      _dataresource = item.setting.dataresource || ''
    }

    let _search = joinMainSearchkey(searches)
    if (item.setting.queryType === 'statistics') {
      _search = ''
    }

    if (/\s/.test(_dataresource)) {
      _dataresource = '(' + _dataresource + ') tb'
    }

    item.setting.dataresource = _dataresource
    item.setting.customScript = _customScript

    let custompage = /@pageSize@|@orderBy@/i.test(_dataresource + _customScript)

    if (_dataresource) {
      if (custompage) {
        _dataresource = `/*system_query*/select ${arr_field} from ${_dataresource} ${_search} `
      } else if (item.setting.laypage === 'true' && item.setting.order) {
        _dataresource = `/*system_query*/select top 10 ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by ${item.setting.order}) as rows from ${_dataresource} ${_search}) tmptable where rows > 0 order by tmptable.rows `
      } else if (item.setting.order) {
        _dataresource = `/*system_query*/select ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by ${item.setting.order}) as rows from ${_dataresource} ${_search}) tmptable order by tmptable.rows `
      } else {
        _dataresource = `/*system_query*/select ${arr_field} from ${_dataresource} ${_search} `
      }
    }

    let sql = ''
    if (_customScript) {
      sql = `declare @ErrorCode nvarchar(50),@retmsg nvarchar(4000),@UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100) select @ErrorCode='',@retmsg =''
        ${_customScript}
        ${_dataresource}
        ${_tailScript}
        aaa:
        if @ErrorCode!=''
          insert into tmp_err_retmsg (ID, ErrorCode, retmsg, CreateUserID) select '1949-10-01 15:00:00',@ErrorCode, @retmsg,'1949-10-01 15:00:00'
      `
    } else if (_tailScript) {
      sql = `declare @ErrorCode nvarchar(50),@retmsg nvarchar(4000),@UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100) select @ErrorCode='',@retmsg =''
        ${_dataresource}
        ${_tailScript}
        aaa:
        if @ErrorCode!=''
          insert into tmp_err_retmsg (ID, ErrorCode, retmsg, CreateUserID) select '1949-10-01 15:00:00',@ErrorCode, @retmsg,'1949-10-01 15:00:00' 
      `
    } else {
      sql = `declare @ErrorCode nvarchar(50),@retmsg nvarchar(4000),@UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100) select @ErrorCode='',@retmsg =''
        ${_dataresource}`
    }
    
    regs.forEach(cell => {
      sql = sql.replace(cell.reg, cell.value)
    })

    let regoptions = getSearchRegs(searches)
    regoptions.push(
      { reg: /@orderBy@/ig, value: item.setting.order || '' },
      { reg: /@pageSize@/ig, value: 9999 },
      { reg: /@pageIndex@/ig, value: 1},
      { reg: /@ID@/ig, value: `''`},
      { reg: /@BID@/ig, value: `''`},
      { reg: /@LoginUID@/ig, value: `'${sessionStorage.getItem('LoginUID') || ''}'`},
      { reg: /@SessionUid@/ig, value: `'${localStorage.getItem('SessionUid') || ''}'`},
      { reg: /@UserID@/ig, value: `'${sessionStorage.getItem('UserID') || ''}'`},
      { reg: /@Appkey@/ig, value: `'${window.GLOB.appkey || ''}'`},
      { reg: /@typename@/ig, value: `'admin'`},
    )

    if (item.type === 'calendar') {
      regoptions.push({ reg: /@mk_year@/ig, value: '2023' })
    }

    regoptions.forEach(cell => {
      sql = sql.replace(cell.reg, cell.value)
    })
    
    return sql
  }

  continue = () => {
    this.setState({status: 'continue'}, () => {
      this.roopSql()
    })
  }

  roopSql = () => {
    if (this.verSqls.length === 0) {
      this.setState({status: 'over'})
      return
    }

    if (this.state.status === 'stop') return

    let item = this.verSqls.shift()

    this.setState({status: this.verSqls.length > 0 ? 'loading' : 'over', execId: item.uuid})

    console.info(`/* 组件：${item.name}  检验项：${item.label} */`)

    Api.sDebug(item.sql).then(result => {
      if (result.status || result.ErrCode === '-2') {
        this.setState({successIds: [...this.state.successIds, item.uuid], execId: ''})
        if (this.verSqls.length > 0) {
          setTimeout(() => {
            this.roopSql()
          }, 100)
        }
      } else {
        let errorMsg = fromJS(this.state.errorMsg).toJS()
        errorMsg[item.uuid] = result.message

        this.setState({
          execId: '',
          status: this.verSqls.length > 0 ? 'error' : 'over',
          errorIds: [...this.state.errorIds, item.uuid],
          errorMsg: errorMsg
        })

        Modal.error({
          title: result.message
        })
      }
    })
  }

  stop = () => {
    this.setState({status: 'stop'})
  }

  over = () => {
    this.verSqls = []
    this.setState({status: 'over', visible: false})
  }

  render() {
    const { visible, sqlList, status, successIds, errorIds, execId, errorMsg } = this.state

    return (
      <>
        <Button className="mk-border-yellow" icon="bug" onClick={this.trigger}>调试</Button>
        <Drawer
          className="debug-modal"
          width={600}
          visible={visible}
          closable={false}
          placement="left"
          destroyOnClose
        >
          <div className="mk-sql-wrap">
            {/* <div className="header">
              <div className="sql-item">
                <div className="sql-1">组件</div>
                <div className="sql-2">检验项</div>
                <div className="sql-3">其他</div>
                <div className="sql-4">状态</div>
              </div>
            </div> */}
            <div className="body">
              {sqlList.map(item => {
                let other = ''
                if (item.supName) {
                  other = `上级：${item.supName}-${item.tabName}`
                }

                let status = <ClockCircleOutlined />

                if (successIds.includes(item.uuid)) {
                  status = <CheckCircleOutlined />
                } else if (errorIds.includes(item.uuid)) {
                  if (errorMsg[item.uuid]) {
                    status = <Tooltip placement="topLeft" title={errorMsg[item.uuid]}>
                      <CloseCircleOutlined />
                    </Tooltip>
                  } else {
                    status = <CloseCircleOutlined />
                  }
                } else if (item.uuid === execId) {
                  status = <LoadingOutlined />
                }

                return <div className="sql-item" key={item.uuid}>
                  <div className="sql-1" title={item.name}><div>{item.name}</div></div>
                  <div className="sql-2" title={item.label}><div>{item.label}</div></div>
                  <div className="sql-3" title={other}><div>{other}</div></div>
                  <div className="sql-4">{status}</div>
                </div>
              })}
            </div>
          </div>
          <div className="mk-footer">
            {status === 'error' ? <Button key="skip" className="mk-border-green" onClick={this.roopSql}>
              跳过
            </Button> : null}
            {status === 'loading' ? <Button key="stop" className="mk-border-orange" onClick={this.stop}>
              暂停
            </Button> : null}
            {status === 'stop' ? <Button key="continue" className="mk-border-orange" onClick={this.continue}>
              继续
            </Button> : null}
            <Button key="colse" className="mk-border-danger" onClick={this.over}>
              关闭
            </Button>
          </div>
        </Drawer>
      </>
    )
  }
}

export default DebugSql